//
//  s95160_1.c	Layer 1 (Physical device interface)
//  Graphic LCD module "SUCCESS ELECTRONIC S95160" I/O library
//  Copyright(C)2013 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.01.22 v0.0  First cording
//

#include "stm32f4xx_gpio.h"				// Public library
#include "s95160_1.h"

//
//	I/O Port define
//
const struct {
	GPIO_TypeDef*		bus1_gpio;
	GPIO_InitTypeDef	bus1_w;
	GPIO_InitTypeDef	bus1_r;

	GPIO_TypeDef*		bus2_gpio;
	GPIO_InitTypeDef	bus2_w;
	GPIO_InitTypeDef	bus2_r;

	GPIO_TypeDef*		cs_gpio;
	GPIO_InitTypeDef	cs;

	GPIO_TypeDef*		rs_gpio;
	GPIO_InitTypeDef	rs;

	GPIO_TypeDef*		wr_gpio;
	GPIO_InitTypeDef	wr;

	GPIO_TypeDef*		rd_gpio;
	GPIO_InitTypeDef	rd;
} lcd_port = {
	LCD_CONF_DB1_GPIO,
	{LCD_CONF_DB1_PIN, GPIO_Mode_OUT,GPIO_Speed_25MHz, GPIO_OType_PP, GPIO_PuPd_NOPULL},
	{LCD_CONF_DB1_PIN, GPIO_Mode_IN, GPIO_Speed_25MHz, GPIO_OType_PP, GPIO_PuPd_UP    },

	LCD_CONF_DB2_GPIO,
	{LCD_CONF_DB2_PIN, GPIO_Mode_OUT,GPIO_Speed_25MHz, GPIO_OType_PP, GPIO_PuPd_NOPULL},
	{LCD_CONF_DB2_PIN, GPIO_Mode_IN, GPIO_Speed_25MHz, GPIO_OType_PP, GPIO_PuPd_UP    },

	LCD_CONF_CS_GPIO,
	{LCD_CONF_CS_PIN, GPIO_Mode_OUT, GPIO_Speed_25MHz, GPIO_OType_PP, GPIO_PuPd_NOPULL},

	LCD_CONF_RS_GPIO,
	{LCD_CONF_RS_PIN, GPIO_Mode_OUT, GPIO_Speed_25MHz, GPIO_OType_PP, GPIO_PuPd_NOPULL},

	LCD_CONF_WR_GPIO,
	{LCD_CONF_WR_PIN, GPIO_Mode_OUT, GPIO_Speed_25MHz, GPIO_OType_PP, GPIO_PuPd_NOPULL},

	LCD_CONF_RD_GPIO,
	{LCD_CONF_RD_PIN, GPIO_Mode_OUT, GPIO_Speed_25MHz, GPIO_OType_PP, GPIO_PuPd_NOPULL}
};


//
//	Open
//
void lcd_portStart(void) {
	LCD_CONF_RCCINIT();

	GPIO_Init(lcd_port.bus1_gpio, (GPIO_InitTypeDef *)&(lcd_port.bus1_w));
	GPIO_Init(lcd_port.bus2_gpio, (GPIO_InitTypeDef *)&(lcd_port.bus2_w));
	GPIO_Init(lcd_port.cs_gpio,   (GPIO_InitTypeDef *)&(lcd_port.cs));
	GPIO_Init(lcd_port.rs_gpio,   (GPIO_InitTypeDef *)&(lcd_port.rs));
	GPIO_Init(lcd_port.wr_gpio,   (GPIO_InitTypeDef *)&(lcd_port.wr));
	GPIO_Init(lcd_port.rd_gpio,   (GPIO_InitTypeDef *)&(lcd_port.rd));

    GPIO_SetBits(lcd_port.cs_gpio, lcd_port.cs.GPIO_Pin);
    GPIO_SetBits(lcd_port.rs_gpio, lcd_port.rs.GPIO_Pin);
    GPIO_SetBits(lcd_port.wr_gpio, lcd_port.wr.GPIO_Pin);
    GPIO_SetBits(lcd_port.rd_gpio, lcd_port.rd.GPIO_Pin);

	return;
}

//
// 	Port I/O
//
void lcd_portWrite(char dc, unsigned short data) {

	GPIO_Write(lcd_port.bus1_gpio, (GPIO_ReadOutputData(lcd_port.bus1_gpio) & ~lcd_port.bus1_w.GPIO_Pin) | (data & lcd_port.bus1_w.GPIO_Pin));
	GPIO_Write(lcd_port.bus2_gpio, (GPIO_ReadOutputData(lcd_port.bus2_gpio) & ~lcd_port.bus2_w.GPIO_Pin) | (data & lcd_port.bus2_w.GPIO_Pin));

	GPIO_WriteBit(lcd_port.rs_gpio, lcd_port.rs.GPIO_Pin, dc);
	GPIO_ResetBits(lcd_port.wr_gpio, lcd_port.wr.GPIO_Pin);
	GPIO_ResetBits(lcd_port.cs_gpio, lcd_port.cs.GPIO_Pin);

	GPIO_SetBits(lcd_port.cs_gpio, lcd_port.cs.GPIO_Pin);
	GPIO_SetBits(lcd_port.wr_gpio, lcd_port.wr.GPIO_Pin);
	return;
}

unsigned short lcd_portRead(char dc) {
	unsigned short	data;

	GPIO_Init(lcd_port.bus1_gpio, (GPIO_InitTypeDef *)&lcd_port.bus1_r);
	GPIO_Init(lcd_port.bus2_gpio, (GPIO_InitTypeDef *)&lcd_port.bus2_r);

	GPIO_WriteBit(lcd_port.rs_gpio, lcd_port.rs.GPIO_Pin, dc);
    GPIO_ResetBits(lcd_port.rd_gpio, lcd_port.rd.GPIO_Pin);
    GPIO_ResetBits(lcd_port.cs_gpio, lcd_port.cs.GPIO_Pin);

	data  = GPIO_ReadInputData(lcd_port.bus1_gpio) & lcd_port.bus1_r.GPIO_Pin;
	data |= GPIO_ReadInputData(lcd_port.bus2_gpio) & lcd_port.bus2_r.GPIO_Pin;

	GPIO_SetBits(lcd_port.cs_gpio, lcd_port.cs.GPIO_Pin);
    GPIO_SetBits(lcd_port.rd_gpio, lcd_port.rd.GPIO_Pin);

	GPIO_Init(lcd_port.bus1_gpio, (GPIO_InitTypeDef *)&lcd_port.bus1_w);
	GPIO_Init(lcd_port.bus2_gpio, (GPIO_InitTypeDef *)&lcd_port.bus2_w);

	return(data);
}

