/*
 * sample4.c	Analog clock
 *
 *  Created on: 2014/02/15
 *      Author: togashitoyohiko
 */
//
//	Segment function, Dynamic figure data
//

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <sys/time.h>
#include "rtc.h"
#include "lcd_gui.h"

enum changeType {CHG_YEAR, CHG_MON, CHG_DAY, CHG_HOUR, CHG_MIN, CHG_SEC, CHG_TIME};
void changeDateTime(enum changeType type, int val) {
	struct tm		tm;
	struct timeval	tp;

	gettimeofday(&tp, NULL);
	localtime_r(&tp.tv_sec, &tm);
	switch(type){
	case CHG_YEAR:
		tm.tm_year = val - 1900;
		break;
	case CHG_MON:
		tm.tm_mon = val;
		break;
	case CHG_DAY:
		tm.tm_mday = val;
		break;
	case CHG_HOUR:
		tm.tm_hour = val;
		break;
	case CHG_MIN:
		tm.tm_min = val;
		break;
	case CHG_SEC:
		tm.tm_sec = val;
		break;
	case CHG_TIME:
		tm.tm_hour = val / 10000;
		tm.tm_min  = (val / 100) % 100;
		tm.tm_sec  = val % 100;
		break;
	default:
		break;
	}
	tp.tv_sec = mktime(&tm);
	settimeofday(&tp, NULL);
}
void yearChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_YEAR,lcd_guiGetIntegerFieldValue(seg));
	return;
}
void monChange(struct lcd_guiPullDown *seg){
	changeDateTime(CHG_MON, lcd_guiGetPullDownSelect(seg));
	return;
}
void dayChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_DAY, lcd_guiGetIntegerFieldValue(seg));
	return;
}
void hourChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_HOUR, lcd_guiGetIntegerFieldValue(seg));
	return;
}
void minChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_MIN, lcd_guiGetIntegerFieldValue(seg));
	return;
}
void secChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_SEC, lcd_guiGetIntegerFieldValue(seg));
	return;
}

//
//	Main program
//
int main(void){
	// Root segment
	struct lcd_guiRoot	*root;

	// Input area
	char *mon[] = {"1","2","3","4","5","6","7","8","9","10","11","12"};
	struct lcd_guiGroup			*input1;
	struct lcd_guiIntegerField	*yy;
	struct lcd_guiPullDown		*mm;
	struct lcd_guiIntegerField	*dd;
	struct lcd_guiIntegerField	*hh;
	struct lcd_guiIntegerField	*mi;
	struct lcd_guiIntegerField	*ss;

	// Analog clock parts
	struct lcd_guiGroup	*analogClock;
	struct lcd_guiLine	*shortNeedle;
	struct lcd_guiLine	*longNeedle;
	struct lcd_guiLine	*secNeedle;

	// etc
	time_t		tBefore;
	time_t		t;
	struct tm	tm;
	double		a;

	rtc_open();
	lcd_drawOpen();
	printf("Start\n");
	root = lcd_guiNewRoot(LCD_COLOR_BLACK, NULL, NULL);
	analogClock = lcd_guiNewGroup(root, 70,70, 100,100, LCD_COLOR_BLUE, NULL, NULL);
	lcd_guiNewRect(analogClock,  0, 0, 100,  1, LCD_COLOR_WHITE);
	lcd_guiNewRect(analogClock,  0, 0,   1,100, LCD_COLOR_WHITE);
	lcd_guiNewRect(analogClock,  0,99, 100,  1, LCD_COLOR_WHITE);
	lcd_guiNewRect(analogClock, 99, 0,   1,100, LCD_COLOR_WHITE);
	shortNeedle = lcd_guiNewLine(analogClock, 50,50,  50, 99, 3, LCD_COLOR_WHITE);
	longNeedle  = lcd_guiNewLine(analogClock, 50,50,  50, 99, 1, LCD_COLOR_WHITE);
	secNeedle   = lcd_guiNewLine(analogClock, 50,50,  50, 99, 1, LCD_COLOR_RED);
	input1 = lcd_guiNewGroup(root, 20,230, 200,40, LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY), NULL, NULL);
	lcd_guiNewStr(input1, 45,16,LCD_COLOR_LIME,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),"-");
	lcd_guiNewStr(input1, 70,16,LCD_COLOR_LIME,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),"-");
	lcd_guiNewStr(input1,132,16,LCD_COLOR_LIME,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),":");
	lcd_guiNewStr(input1,158,16,LCD_COLOR_LIME,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),":");
	lcd_guiNewStr(input1, 10, 2,LCD_COLOR_DARKGRAY,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),"Date");
	lcd_guiNewStr(input1,113, 2,LCD_COLOR_DARKGRAY,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),"Time");
	yy = lcd_guiNewIntegerField(input1,  10,10,  35,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%d",   LCD_FONT_8x12, 1970,2199, 2014, yearChange);
	mm = lcd_guiNewPullDown(    input1,  51,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, mon,    LCD_FONT_8x12, 12, -1, monChange);
	dd = lcd_guiNewIntegerField(input1, 76,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY,  "%d",   LCD_FONT_8x12, 1,31, 1, dayChange);
	hh = lcd_guiNewIntegerField(input1, 113,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%d",   LCD_FONT_8x12, 0,23, 0, hourChange);
	mi = lcd_guiNewIntegerField(input1, 139,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%02d", LCD_FONT_8x12, 0,59, 0, minChange);
	ss = lcd_guiNewIntegerField(input1, 164,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%02d", LCD_FONT_8x12, 0,59, 0, secChange);

	tBefore = 0;
	for(;;){
		t = time(NULL);
		if ((tBefore != t) && (root->popup == NULL)) {
			tBefore = t;
			localtime_r(&t, &tm);

			a = M_PI * -2.0 / (12*60) * (tm.tm_hour * 60 + tm.tm_min + (6*60));
			shortNeedle->endPoint.x = sin(a) * 30 + 50;
			shortNeedle->endPoint.y = cos(a) * 30 + 50;
			a = M_PI * -2.0 / 60 * (tm.tm_min + 30);
			longNeedle->endPoint.x = sin(a) * 40 + 50;
			longNeedle->endPoint.y = cos(a) * 40 + 50;
			a = M_PI * -2.0 / 60 * (tm.tm_sec + 30);
			secNeedle->endPoint.x = sin(a) * 40 + 50;
			secNeedle->endPoint.y = cos(a) * 40 + 50;
			lcd_guiSetDirtys(analogClock);

			lcd_guiSetIntegerFieldValue(yy, tm.tm_year+1900);
			lcd_guiSetPullDownSelect(mm, tm.tm_mon);
			lcd_guiSetIntegerFieldValue(dd, tm.tm_mday);
			lcd_guiSetIntegerFieldValue(hh, tm.tm_hour);
			lcd_guiSetIntegerFieldValue(mi, tm.tm_min);
			lcd_guiSetIntegerFieldValue(ss, tm.tm_sec);
		}
		lcd_guiHandler(root);
	}
	return(0);
}
