/*
 * sample3.c	Clock and Calendar
 *
 *  Created on: 2014/02/17
 *      Author: togashitoyohiko
 */
//
//	Segment function, Static figure data
//

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <sys/time.h>
#include "rtc.h"
#include "lcd_gui.h"

enum changeType {CHG_YEAR, CHG_MON, CHG_DAY, CHG_HOUR, CHG_MIN, CHG_SEC, CHG_TIME};
void changeDateTime(enum changeType type, int val) {
	struct tm		tm;
	struct timeval	tp;

	gettimeofday(&tp, NULL);
	localtime_r(&tp.tv_sec, &tm);
	switch(type){
	case CHG_YEAR:
		tm.tm_year = val - 1900;
		break;
	case CHG_MON:
		tm.tm_mon = val;
		break;
	case CHG_DAY:
		tm.tm_mday = val;
		break;
	case CHG_HOUR:
		tm.tm_hour = val;
		break;
	case CHG_MIN:
		tm.tm_min = val;
		break;
	case CHG_SEC:
		tm.tm_sec = val;
		break;
	case CHG_TIME:
		tm.tm_hour = val / 10000;
		tm.tm_min  = (val / 100) % 100;
		tm.tm_sec  = val % 100;
		break;
	default:
		break;
	}
	tp.tv_sec = mktime(&tm);
	settimeofday(&tp, NULL);
}
void yearChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_YEAR,lcd_guiGetIntegerFieldValue(seg));
	return;
}
void monChange(struct lcd_guiPullDown *seg){
	changeDateTime(CHG_MON, lcd_guiGetPullDownSelect(seg));
	return;
}
void dayChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_DAY, lcd_guiGetIntegerFieldValue(seg));
	return;
}
void hourChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_HOUR, lcd_guiGetIntegerFieldValue(seg));
	return;
}
void minChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_MIN, lcd_guiGetIntegerFieldValue(seg));
	return;
}
void secChange(struct lcd_guiIntegerField *seg){
	changeDateTime(CHG_SEC, lcd_guiGetIntegerFieldValue(seg));
	return;
}

//
//	Graphic figure data
//
// Input area
char *mon[] = {"1","2","3","4","5","6","7","8","9","10","11","12"};
struct lcd_guiIntegerFieldSet	ss = LCD_GUI_INIT_INTEGER_FIELD_SET(ss, NULL, NULL, 164,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%02d", LCD_FONT_8x12, 0,59, 0, secChange);
struct lcd_guiIntegerFieldSet	mi = LCD_GUI_INIT_INTEGER_FIELD_SET(mi, &ss,  NULL, 139,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%02d", LCD_FONT_8x12, 0,59, 0, minChange);
struct lcd_guiIntegerFieldSet	hh = LCD_GUI_INIT_INTEGER_FIELD_SET(hh, &mi,  NULL, 113,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%d",   LCD_FONT_8x12, 0,23, 0, hourChange);
struct lcd_guiIntegerFieldSet	dd = LCD_GUI_INIT_INTEGER_FIELD_SET(dd, &hh,  NULL, 76,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%d",   LCD_FONT_8x12, 1,31, 1, dayChange);
struct lcd_guiPullDownSet		mm = LCD_GUI_INIT_PULL_DOWN_SET(    mm, &dd,  NULL,  51,10,  19,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, mon,    LCD_FONT_8x12, 12, -1, monChange);
struct lcd_guiIntegerFieldSet	yy = LCD_GUI_INIT_INTEGER_FIELD_SET(yy, &mm,  NULL,  10,10,  35,20, LCD_COLOR_LIME, LCD_COLOR_GRAY, "%d",   LCD_FONT_8x12, 1970,2199, 2014, yearChange);
struct lcd_guiStr				s6 = LCD_GUI_INIT_STR(&yy,NULL,113, 2,LCD_COLOR_DARKGRAY,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),"Time");
struct lcd_guiStr				s5 = LCD_GUI_INIT_STR(&s6,NULL, 10, 2,LCD_COLOR_DARKGRAY,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),"Date");
struct lcd_guiStr				s4 = LCD_GUI_INIT_STR(&s5,NULL,158,16,LCD_COLOR_LIME,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),":");
struct lcd_guiStr				s3 = LCD_GUI_INIT_STR(&s4,NULL,132,16,LCD_COLOR_LIME,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),":");
struct lcd_guiStr				s2 = LCD_GUI_INIT_STR(&s3,NULL, 70,16,LCD_COLOR_LIME,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),"-");
struct lcd_guiStr				s1 = LCD_GUI_INIT_STR(&s2,NULL, 45,16,LCD_COLOR_LIME,LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY),"-");
struct lcd_guiGroupSet			input1 = LCD_GUI_INIT_GROUP_SET(input1, NULL, NULL, 20,230, 200,40, LCD_COLOR_TO_DARK(LCD_COLOR_DIMGRAY), &s1, NULL);

// Analog clock parts
struct lcd_guiLine      secNeedle   = LCD_GUI_INIT_LINE(NULL,        NULL, 50,50,  50, 99, 1, LCD_COLOR_RED);
struct lcd_guiLine      longNeedle  = LCD_GUI_INIT_LINE(&secNeedle,  NULL, 50,50,  50, 99, 1, LCD_COLOR_WHITE);
struct lcd_guiLine      shortNeedle = LCD_GUI_INIT_LINE(&longNeedle, NULL, 50,50,  50, 99, 3, LCD_COLOR_WHITE);
struct lcd_guiRect		frame4      = LCD_GUI_INIT_RECT(&shortNeedle,NULL, 99, 0,   1,100, LCD_COLOR_WHITE);
struct lcd_guiRect		frame3      = LCD_GUI_INIT_RECT(&frame4,     NULL,  0,99, 100,  1, LCD_COLOR_WHITE);
struct lcd_guiRect		frame2      = LCD_GUI_INIT_RECT(&frame3,     NULL,  0, 0,   1,100, LCD_COLOR_WHITE);
struct lcd_guiRect		frame1      = LCD_GUI_INIT_RECT(&frame2,     NULL,  0, 0, 100,  1, LCD_COLOR_WHITE);
struct lcd_guiGroupSet	analogClock = LCD_GUI_INIT_GROUP_SET(analogClock, &input1, NULL, 70,70, 100,100, LCD_COLOR_BLUE, &frame1, NULL);

// Root segment
struct lcd_guiRootSet	root = LCD_GUI_INIT_ROOT_SET(root, LCD_COLOR_BLACK, &analogClock, NULL);

//
//	Main program
//
int main(void){
	// etc
	time_t			tBefore;
	time_t			t;
	struct tm		tm;
	double			a;

	rtc_open();
	lcd_drawOpen();
	printf("Start\n");
	lcd_guiSetRelink((struct lcd_gui *)&root);
	tBefore = 0;
	for(;;){
		t = time(NULL);
		if ((tBefore != t) && (root.root.popup == NULL)) {
			tBefore = t;
			localtime_r(&t, &tm);

			a = M_PI * -2.0 / (12*60) * (tm.tm_hour * 60 + tm.tm_min + (6*60));
			shortNeedle.endPoint.x = sin(a) * 30 + 50;
			shortNeedle.endPoint.y = cos(a) * 30 + 50;
			a = M_PI * -2.0 / 60 * (tm.tm_min + 30);
			longNeedle.endPoint.x = sin(a) * 40 + 50;
			longNeedle.endPoint.y = cos(a) * 40 + 50;
			a = M_PI * -2.0 / 60 * (tm.tm_sec + 30);
			secNeedle.endPoint.x = sin(a) * 40 + 50;
			secNeedle.endPoint.y = cos(a) * 40 + 50;
			lcd_guiSetDirtys(&analogClock);

			lcd_guiSetIntegerFieldValue(&yy.integerField, tm.tm_year+1900);
			lcd_guiSetPullDownSelect(&mm.pullDown, tm.tm_mon);
			lcd_guiSetIntegerFieldValue(&dd.integerField, tm.tm_mday);
			lcd_guiSetIntegerFieldValue(&hh.integerField, tm.tm_hour);
			lcd_guiSetIntegerFieldValue(&mi.integerField, tm.tm_min);
			lcd_guiSetIntegerFieldValue(&ss.integerField, tm.tm_sec);
		}
		lcd_guiHandler(&root.root);
	}
	return(0);
}
