//
//  rtc.c	Interrupt base Real time clock
//  Copyright(C)2013-2014 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.05.22 v0.0  First cording
//    2014.03.05 v0.1  gettimeofday()
//
//
#include <sys/time.h>
#include <stddef.h>
#include <errno.h>

#include <stdint.h>						// Avoided bug for compile error stm32f4xx.h
#include "stm32f4xx_tim.h"
#include "stm32f4xx_rcc.h"

//
//	Global area
//
volatile time_t		rtc_time;

//
//	House keeping
//
void rtc_open(void) {
	NVIC_InitTypeDef		n;
	TIM_TimeBaseInitTypeDef	t;

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);

	n.NVIC_IRQChannel                   = TIM4_IRQn;
	n.NVIC_IRQChannelPreemptionPriority = 0;
	n.NVIC_IRQChannelSubPriority        = 1;
	n.NVIC_IRQChannelCmd                = ENABLE;
	NVIC_Init(&n);

	TIM_InternalClockConfig(TIM4);
	t.TIM_Period        = 10000;	// 100us * 10000 = 1s
	t.TIM_Prescaler     = ((SystemCoreClock / 2) / 10000) - 1;	// 10kHz:100us;
	t.TIM_ClockDivision = 0;
	t.TIM_CounterMode   = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM4, &t);
	TIM_ITConfig(TIM4, TIM_IT_Update, ENABLE);
	TIM_Cmd(TIM4, ENABLE);
	return;
}

void rtc_close(void){
	TIM_DeInit(TIM4);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, DISABLE);
	return;
}

#ifdef INSERT_TO_IRQ					// stm32f4xx_it.c
#include <sys/time.h>
void TIM4_IRQHandler(void){
	extern volatile time_t	rtc_time;
	rtc_time++;
	return;
}
#endif

//
//	Unix system call
//
#ifndef DEL
time_t time(time_t *t) {
	if (t != NULL) {
		*t = rtc_time;
	}
	return(rtc_time);
}
#endif

int settimeofday(const struct timeval *tv, const struct timezone *tz){
	int	r;

	if (tv != NULL) {
		rtc_time = tv->tv_sec;
		TIM_SetCounter(TIM4, tv->tv_usec / 10);
		r = 0;
	} else {
		r = -1;
		errno = EFAULT;
	}
	return(r);
}

int gettimeofday(struct timeval *tp, void *tzp){
	struct timezone	*tz;

	tz = tzp;
	if (tp != NULL) {
		tp->tv_sec  = rtc_time;
		tp->tv_usec = TIM_GetCounter(TIM4) * 10;
	}
	if (tz != NULL) {
		tz->tz_minuteswest = 0;
		tz->tz_dsttime     = 0;
	}
	return(0);
}
